from flask import Flask
from ..database_util import init_db
from ..routes import init_api
from flask_script import Manager
from flask_migrate import Migrate, MigrateCommand






def create_app(appname,blueprint,config):
    app = Flask(appname)
    app.register_blueprint(blueprint)
    app.config.from_object(config)
    migrate = init_db(app)
    init_api(app)
    manager = Manager(app=app)
    manager.add_command('db', MigrateCommand)
    return app, migrate, manager


class DevelopmentConfig(): 
    DEBUG = True
    SQLALCHEMY_DATABASE_URI = "mysql+pymysql://root:123456@127.0.0.1/User"
    SQLALCHEMY_TRACK_MODIFICATIONS = False
    MIGRATE = True

class ProductionConfig(): 
    DEBUG = True
    SQLALCHEMY_DATABASE_URI = "mysql+pymysql://root:123456@127.0.0.1/User"
    SQLALCHEMY_TRACK_MODIFICATIONS = False
    MIGRATE = False


from flask import Flask
from .routes import blue
from .config import DevelopmentConfig, ProductionConfig
from .factory import create_app

app, migrate, manager = create_app(appname=__name__, blueprint=blue, config=ProductionConfig)

def main():
    if app.config["MIGRATE"] == True:
        manager.run()
    else:
        app.run(debug=False, host='0.0.0.0', port=9888)


if __name__ == "__main__":
    main()



from ..version import version_num
from flask import Blueprint, jsonify
from ..database_util import db, User
import uuid
from flask import request, make_response, abort, Response
from ..helper import get_user, authorized, auth_required
import datetime
blue = Blueprint(__name__, import_name="blue")


# This part below, initialize the api for the real route, the root route of / is still loaded by the default blueprint
from flask_restx import Api, Resource, Namespace, fields, reqparse
api = Api(title="PPEService RestApi Documentation",version=version_num,doc="/api/docs")
ns = Namespace('/',"PPEService RestApi Documentation")
def init_api(app):
    api.init_app(app=app)
    api.add_namespace(ns)
    
    
@blue.route('/')
def index():
    return "Test Passed"

@ns.route('/initdb')
class initdb(Resource):
    def post(self):
        """
        If you want to re create the database, you could run this, 
        this function shouldb't be exposed externally
        """
        db.create_all()
        today = datetime.date.today()
        admin = User(token=str(uuid.uuid4()), username='admin',
                    password='admin@example.com',token_generated_date=today)
        db.session.add(admin)
        db.session.commit()
        return "Operation done!"

# Below are the orm object structures
account = api.model('account_structure', {
    'username': fields.String(required=True, description='username'),
    'password': fields.String(required=True, description='password')
})

# token = api.model('token_structure', {
#     'token': fields.String(required=True, description='username'),
# })
def RequestParser():
    parser = reqparse.RequestParser()
    parser.add_argument('token', required=True, location='headers')
    return parser

@ns.route('/signup')
@ns.expect(account)
class signup(Resource):
    def post(self):
        """
        This api allows you to signup
        """
        try:
            today = datetime.date.today()
            req_data = request.get_json(force=True)
            username = req_data['username']
            password = req_data['password']
            if get_user(username):
                return "Username already exists"
            token = str(uuid.uuid4())
            new_user = User(token=token, username=username, password=password, token_generated_date=today)
            db.session.add(new_user)
            db.session.commit()
            # param today is in type of date, but the date from database is in type of datetime
            # so need to do the convertion, we can convert the datetime to date to do the comparasion
            # print(today == User.query.filter_by(username=username).first().token_generated_date.date())

            ret = {"msg": "Signed up successfully, you can login now!"}
            return jsonify(ret)
        except Exception as e:
            return "Signup error: " + str(e)

@ns.route('/login')
@ns.expect(account)
class login(Resource):
    def post(self):
        """
        This api allows you to login
        """
        try:
            req_data = request.get_json(force=True)
            username = req_data['username']
            password = req_data['password']
            # this is an database object
            user = get_user(username)

            if user.password == password:
                token = str(user.token)
                today = datetime.date.today()
                user.token_generated_date = today
                db.session.commit()
                ret = jsonify({"msg": "Login success", "token":token})
                resp = make_response(ret)
                return resp
            return "Login failed"
        except Exception as e:
            return "Login errod: " + str(e)

# This is the token structre, as it's from the header, so should be retrieved from parser
token_parser = RequestParser()


@ns.route('/logout')
@ns.expect(token_parser)
class logout(Resource):
    @auth_required
    def post(self, token):
        """
        This api allows you to logout
        """
        try:
            user = User.query.filter_by(token=token).first()
            resp = make_response("Logged out")
            today = datetime.date.today()
            user.token_generated_date = today - datetime.timedelta(days=5)
            db.session.commit()
            return resp
        except Exception as e:
            return "Logour error: " + str(e)

@ns.route('/user')
@ns.expect(token_parser)
class TestUser(Resource):
    @auth_required
    def get(self, token):
        """
        This is a testing function for the token based authentication, 
        all the function implemented later should be implemented this way"""
        return "hello " + token

# @blue.route('/dropdb', methods=['GET'])
# @auth_required
# def drop(token):
#     return "hello " + token


from flask_sqlalchemy import SQLAlchemy
from flask_migrate import Migrate

db = SQLAlchemy()

def init_db(app):
    db.init_app(app=app)
    migrate = Migrate(app, db)
    return migrate

class User(db.Model):
    token = db.Column(db.String(100), primary_key=True)
    username = db.Column(db.String(100), unique=True, nullable=False)
    password = db.Column(db.String(100), nullable=False)
    token_generated_date = db.Column(db.DateTime, nullable=False)
    # User can comment out this line out to test the database migration result
    # added_col = db.Column(db.String(100), nullable=False)




from flask import request
from ..database_util import User
from functools import wraps
import datetime

def get_user(username):
    """
    get_user will extract the corresponding user database object
    according to the unique userame
    param username: your username
    type: string
    """
    return User.query.filter_by(username=username).first()

def authorized(user, token):
    if user:
        return user.token == token and \
             user.token_generated_date.date() + datetime.timedelta(days=3) > datetime.date.today()
    else:
        return False

def auth_required(function):
    @wraps(function)
    def decorated(*args, **kwargs):
        """
            Method to verify user before making each request
            :return : returns successful route if success else
                        auth failed message
        """
        try:
            print("inside of try")
            token = request.headers['token']
            user = User.query.filter_by(token=token).first()
            if user is not None:
                return function(token=token, *args, **kwargs) if authorized(user, token) else "Auth failed, maybe you token is expired, try login again"
        except Exception as e:
            return "Auth error: " + str(e)
    return decorated



import os
import re
import ast
"""
This is an personal website generator(could be any website or small services generator)
It control build, deploy, domain name generation and all process needed by having a website
"""
from setuptools import setup, find_packages
packages_required = []
with open("requirements.txt", "r") as f:
    for line in f:
        packages_required.append(line[:-1])

version = "0.0.0.0"

with open ('ppe/version.py') as f:
    for line in f:
        version = ast.parse(line).body[0].value.s

setup(
    name='ppeservice',
    version=version,
    license='GPLv3',
    author='Ye Zhang, Junqi Zhang',
    author_email="zhangforchat@gmail.com",
    url="https://github.com/JunqiZhang0/py-production-experiment",
    description='Personal website generator(could be any website or small services generator)',
    long_description=__doc__,
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    install_requires=packages_required,

    classifiers=[
        'Development Status :: 1 - Planning'
    ],
    entry_points={
        'console_scripts': ['ppe=ppe.manage:main']
    }
)
